"use strict";
/**
 * @file This plugin is to object detection data from PASCOL VOC format. Make sure that
 * the data is conform to expectation.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const pipcook_core_1 = require("@pipcook/pipcook-core");
const tf = __importStar(require("@tensorflow/tfjs-node-gpu"));
const jimp_1 = __importDefault(require("jimp"));
const glob_promise_1 = __importDefault(require("glob-promise"));
const path = __importStar(require("path"));
const _cliProgress = require('cli-progress');
const concatenateDataFlows = (fileNames, imgSize, oneHotMap, dataFlows, type) => __awaiter(void 0, void 0, void 0, function* () {
    console.log(`access ${type} image data...`);
    const bar1 = new _cliProgress.SingleBar({}, _cliProgress.Presets.shades_classic);
    bar1.start(fileNames.length, 0);
    for (let j = 0; j < fileNames.length; j++) {
        const jsonData = yield pipcook_core_1.parseAnnotation(fileNames[j]);
        bar1.update(j);
        let image = yield jimp_1.default.read(path.join(jsonData.annotation.folder[0], jsonData.annotation.filename[0]));
        const xratio = 1.0 * imgSize[0] / image.bitmap.width;
        const yratio = 1.0 * imgSize[1] / image.bitmap.height;
        image = image.resize(imgSize[0], imgSize[1]);
        const trainImageBuffer = yield image.getBufferAsync(jimp_1.default.MIME_JPEG);
        const imageArray = new Uint8Array(trainImageBuffer);
        const target = [];
        jsonData.annotation.object.forEach((item) => {
            target.push(oneHotMap[item.name[0]]);
            target.push(parseFloat(item.bndbox[0].xmin[0]) * xratio);
            target.push(parseFloat(item.bndbox[0].xmax[0]) * xratio);
            target.push(parseFloat(item.bndbox[0].ymin[0]) * yratio);
            target.push(parseFloat(item.bndbox[0].ymax[0]) * yratio);
        });
        for (let ii = jsonData.annotation.object.length; ii < 10; ii++) {
            target.push(-1);
            for (let jj = 0; jj < 4; jj++) {
                target.push(0);
            }
        }
        dataFlows.push({
            xs: tf.cast(tf.node.decodeImage(imageArray, 3), 'float32'),
            ys: tf.tensor1d(target)
        });
    }
    bar1.stop();
});
/**
 * merge all possible values of labels. Get the map between label and numeric value
 * @param data
 */
const getLabelMap = (data) => __awaiter(void 0, void 0, void 0, function* () {
    const labelSet = new Set();
    for (let i = 0; i < data.length; i++) {
        const dataItem = data[i];
        const { trainDataPath } = dataItem;
        const trainFileNames = yield glob_promise_1.default(path.join(trainDataPath, '*.xml'));
        for (let j = 0; j < trainFileNames.length; j++) {
            const fileName = trainFileNames[j];
            const imageData = yield pipcook_core_1.parseAnnotation(fileName);
            imageData.annotation.object.forEach((item) => {
                labelSet.add(item.name[0]);
            });
        }
    }
    const labelArray = Array.from(labelSet);
    const oneHotMap = {};
    labelArray.forEach((label, index) => {
        oneHotMap[label] = index;
    });
    return oneHotMap;
});
/**
 * The plugin used to access data from different sources. It will detect all possible values of labels and
 * merge them into numeric expressions.
 * @param data: origin sample data
 * @param imgSize: the image size to uniform
 */
const imageDetectionDataAccess = (data, args) => __awaiter(void 0, void 0, void 0, function* () {
    if (!Array.isArray(data)) {
        data = [data];
    }
    const { imgSize = [224, 224] } = args || {};
    const oneHotMap = yield getLabelMap(data);
    const trainDataFlows = [], validationDataFlows = [], testDataFlows = [];
    for (let i = 0; i < data.length; i++) {
        const dataSample = data[i];
        const { trainDataPath, validationDataPath, testDataPath } = dataSample;
        const trainFileNames = yield glob_promise_1.default(path.join(trainDataPath, '*.xml'));
        yield concatenateDataFlows(trainFileNames, imgSize, oneHotMap, trainDataFlows, 'train data');
        if (validationDataPath) {
            const validationFileNames = yield glob_promise_1.default(path.join(validationDataPath, '*.xml'));
            yield concatenateDataFlows(validationFileNames, imgSize, oneHotMap, validationDataFlows, 'validation data');
        }
        if (testDataPath) {
            const testFileNames = yield glob_promise_1.default(path.join(testDataPath, '*.xml'));
            yield concatenateDataFlows(testFileNames, imgSize, oneHotMap, testDataFlows, 'test data');
        }
    }
    const result = {
        trainData: tf.data.array(trainDataFlows),
        metaData: {
            feature: {
                name: 'xs',
                type: 'float32',
                shape: [imgSize[0], imgSize[1], 3]
            },
            label: {
                name: 'ys',
                type: 'int32',
                shape: [1, Object.keys(oneHotMap).length],
                valueMap: oneHotMap
            },
        }
    };
    if (validationDataFlows.length > 0) {
        result.validationData = tf.data.array(validationDataFlows);
    }
    if (testDataFlows.length > 0) {
        result.testData = tf.data.array(testDataFlows);
    }
    return result;
});
exports.default = imageDetectionDataAccess;
//# sourceMappingURL=index.js.map